from __future__ import absolute_import, division, print_function

from octane.api_resources.abstract.api_resource import APIResource
from octane import api_requestor, util


class CreateableAPIResource(APIResource):
    @classmethod
    def create(
        cls, api_key=None, idempotency_key=None, octane_version=None, **params
    ):
        requestor = api_requestor.APIRequestor(
            api_key, api_version=octane_version
        )
        url = cls.class_url()
        headers = util.populate_headers(idempotency_key)
        response, api_key = requestor.request("post", url, params, headers)

        return util.convert_to_octane_object(response, api_key, octane_version)



import octane

meter = octane.Meter.create(name="num_video_sessions", display_name="Number of Video Sessions", meter_type="counter", is_incremental=True)

price_plan = octane.PricePlan(name="basic_price_plan", display_name="Basic Price Plan", period="month", metered_components=[{"meter_name": "num_video_sessions", "price_scheme": {}}])



from __future__ import absolute_import, division, print_function

import atexit
import os
import sys
from distutils.version import StrictVersion

import pytest

import octane
from octane.six.moves.urllib.request import urlopen
from octane.six.moves.urllib.error import HTTPError

from tests.request_mock import RequestMock


@pytest.fixture(autouse=True)
def setup_octane():
    orig_attrs = {
        "api_base": octane.api_base,
        "api_key": octane.api_key,
        "client_id": octane.client_id,
        "default_http_client": octane.default_http_client,
    }
    http_client = octane.http_client.new_default_http_client()
    octane.api_base = "http://localhost:8080"
    octane.api_key = "sk_test_123"
    octane.client_id = "ca_123"
    octane.default_http_client = http_client
    yield
    http_client.close()
    octane.api_base = orig_attrs["api_base"]
    octane.api_key = orig_attrs["api_key"]
    octane.client_id = orig_attrs["client_id"]
    octane.default_http_client = orig_attrs["default_http_client"]


@pytest.fixture
def request_mock(mocker):
    return RequestMock(mocker)



from __future__ import absolute_import, division, print_function

from octane.api_resources.abstract import CreateableAPIResource
from octane.api_resources.abstract import DeletableAPIResource
from octane.api_resources.abstract import ListableAPIResource
from octane.api_resources.abstract import UpdateableAPIResource


class Meter(
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
):
    OBJECT_NAME = "meter"



from __future__ import absolute_import, division, print_function
from datetime import datetime

from octane.api_resources.abstract import CreateableAPIResource


class Measurement(
    CreateableAPIResource,
):
    OBJECT_NAME = "measurement"

    @classmethod
    def create(
        cls, api_key=None, idempotency_key=None, octane_version=None, **params
    ):
        if "time" not in params:
            params["time"] = datetime.utcnow().isoformat()

        return super(Measurement, cls).create(api_key=api_key, idempotency_key=idempotency_key, octane_version=octane_version, **params)


from __future__ import absolute_import, division, print_function


class ImproperTestObject(Exception):
    pass


class TestOctaneResource(object):
    TEST_RESOURCE_ID = "abc123"

    def _is_listable(self, request_mock):
        method = "get"
        url = self.BASE_URL
        request_mock.stub_request(
            method,
            url,
            {
                "object": "list",
                "data": [
                    {
                        "object": self.RESOURCE_CLASS.OBJECT_NAME,
                        "name": self.TEST_RESOURCE_ID,
                    }
                ],
            },
        )

        resources = self.RESOURCE_CLASS.list()

        request_mock.assert_requested(method, url)
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], self.RESOURCE_CLASS)

    def _is_retrievable(self, request_mock):
        method = "get"
        url = "%s/%s" % (self.BASE_URL, self.TEST_RESOURCE_ID)
        request_mock.stub_request(
            method,
            url,
            {
                "object": self.RESOURCE_CLASS.OBJECT_NAME,
                "name": self.TEST_RESOURCE_ID,
            },
        )

        resource = self.RESOURCE_CLASS.retrieve(self.TEST_RESOURCE_ID)

        request_mock.assert_requested(method, url)
        assert isinstance(resource, self.RESOURCE_CLASS)

    def _is_creatable(self, request_mock):
        method = "post"
        url = self.BASE_URL
        request_mock.stub_request(
            method,
            url,
            {
                "object": self.RESOURCE_CLASS.OBJECT_NAME,
                "name": self.TEST_RESOURCE_ID,
            },
        )

        resource = self.RESOURCE_CLASS.create(name=self.TEST_RESOURCE_ID)

        request_mock.assert_requested("post", self.BASE_URL)
        assert isinstance(resource, self.RESOURCE_CLASS)

    def _is_saveable(self, request_mock):
        method = "put"
        url = "%s/%s" % (self.BASE_URL, self.TEST_RESOURCE_ID)
        request_mock.stub_request(
            method,
            url,
            {
                "object": self.RESOURCE_CLASS.OBJECT_NAME,
                "name": self.TEST_RESOURCE_ID,
            },
        )

        request_mock.stub_request(
            "get",
            "%s/%s" % (self.BASE_URL, self.TEST_RESOURCE_ID),
            {
                "object": self.RESOURCE_CLASS.OBJECT_NAME,
                "name": self.TEST_RESOURCE_ID,
                "key": "old_value",
            },
        )

        resource = self.RESOURCE_CLASS.retrieve(self.TEST_RESOURCE_ID)
        resource.key = "new_value"
        resource.save()

        request_mock.assert_requested(method, url)

    def _is_modifiable(self, request_mock):
        method = "put"
        url = "%s/%s" % (self.BASE_URL, self.TEST_RESOURCE_ID)
        request_mock.stub_request(
            method,
            url,
            {
                "object": self.RESOURCE_CLASS.OBJECT_NAME,
                "name": self.TEST_RESOURCE_ID,
            },
        )

        request_mock.stub_request(
            "get",
            "%s/%s" % (self.BASE_URL, self.TEST_RESOURCE_ID),
            {
                "object": self.RESOURCE_CLASS.OBJECT_NAME,
                "name": self.TEST_RESOURCE_ID,
            },
        )

        _ = self.RESOURCE_CLASS.modify(
            self.TEST_RESOURCE_ID, metadata={"key": "value"}
        )

        request_mock.assert_requested(method, url)

    def _is_deletable(self, request_mock):
        method = "delete"
        url = "%s/%s" % (self.BASE_URL, self.TEST_RESOURCE_ID)
        request_mock.stub_request(method, url)

        request_mock.stub_request(
            "get",
            "%s/%s" % (self.BASE_URL, self.TEST_RESOURCE_ID),
            {
                "object": self.RESOURCE_CLASS.OBJECT_NAME,
                "name": self.TEST_RESOURCE_ID,
            },
        )

        resource = self.RESOURCE_CLASS.retrieve(self.TEST_RESOURCE_ID)
        resource.delete()

        request_mock.assert_requested(method, url)

    def _can_delete(self, request_mock):
        method = "delete"
        url = "%s/%s" % (self.BASE_URL, self.TEST_RESOURCE_ID)
        request_mock.stub_request(method, url)

        self.RESOURCE_CLASS.delete(self.TEST_RESOURCE_ID)

        request_mock.assert_requested(method, url)



from __future__ import absolute_import, division, print_function
from tests.api_resources.test_octane_resource import TestOctaneResource

import octane


class TestOctaneMeter(TestOctaneResource):
    RESOURCE_CLASS = octane.Meter
    BASE_URL = "/meters"

    def test_is_listable(self, request_mock):
        super()._is_listable(request_mock)

    def test_is_retrievable(self, request_mock):
        super()._is_retrievable(request_mock)

    def test_is_creatable(self, request_mock):
        super()._is_creatable(request_mock)

    def test_is_saveable(self, request_mock):
        super()._is_saveable(request_mock)

    def test_is_modifiable(self, request_mock):
        super()._is_modifiable(request_mock)

    def test_is_deletable(self, request_mock):
        super()._is_deletable(request_mock)

    def test_can_delete(self, request_mock):
        super()._can_delete(request_mock)



from __future__ import absolute_import, division, print_function

import os

import octane


octane.api_key = os.environ.get("OCTANE_SECRET_KEY")

print("Attempting to create customer...")

resp = octane.Customer.create(
    name="Customer1",
)

print("Success: %r" % (resp))



from __future__ import absolute_import, division, print_function

from octane.api_resources.abstract import CreateableAPIResource

class Subscription(
    CreateableAPIResource
):
    OBJECT_NAME = "subscription"

    @classmethod
    def create(cls, sid, **params):
        raise NotImplementedError(
            "Can't create a mapping without a customer name. "
            "Use customer.mappings.create()"
        )

    @classmethod
    def retrieve(cls, id, api_key=None, **params):
        raise NotImplementedError("Can't retrieve a subscription.")
